#!/usr/bin/env python
# -*- encoding=utf8 -*-

"""
Author: Hanyu Wang
Created time: 2024-07-23 22:34:21
Last Modified by: Hanyu Wang
Last Modified time: 2024-07-23 22:34:55
"""

import datetime


def writeHeader(f):
    headerString = ""
    headerString += "//" + "-" * 78 + "\n"
    headerString += "// " + "Generated by LeaP compiler" + "\n"
    headerString += "// " + "Author: " + "Hanyu Wang" + "\n"
    headerString += (
        "// " + "Date: " + datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S") + "\n"
    )
    headerString += "//" + "-" * 78 + "\n"
    f.write(headerString)
